@@ -1,14 +1,14 @@
 #
 # This file is part of Config-Model-Approx
 #
-# This software is Copyright (c) 2009-2017 by Dominique Dumont.
+# This software is Copyright (c) 2015-2018 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Backend::Approx ;
-$Config::Model::Backend::Approx::VERSION = '1.010';
+$Config::Model::Backend::Approx::VERSION = '1.011';
 use Mouse ;
 use Log::Log4perl qw(get_logger :levels);
 use Carp ;
@@ -16,19 +16,12 @@ use File::Copy ;
 use File::Path ;
 use 5.010 ;
 
-
 extends 'Config::Model::Backend::Any';
 
-# optional
-sub suffix { 
-   return '.conf';
-}
-
 sub annotation {
-    return 1 ; 
+    return 1 ;
 }
 
-
 my $logger = Log::Log4perl::get_logger('Backend::Approx');
 
 sub read {
@@ -43,13 +36,11 @@ sub read {
     # io_handle  => $io           # IO::File object
     # check      => yes|no|skip
 
-    die "Cannot read $args{config_dir}$args{file}\n" unless defined $args{io_handle} ;
-
     $logger->info("loading config file $args{file}") if defined $args{file};
-    my @lines = $args{io_handle}->getlines ;
+    my @lines = $args{file_path}->lines_utf8 ;
     my $global = $self->read_global_comments(\@lines, '#') ;
     $self->node->annotation($global) ;
-    
+
     my @data = $self->associates_comments_with_data(\@lines, '#') ;
 
     foreach my $item (@data) {
@@ -57,12 +48,12 @@ sub read {
 
         my ($k,$v) = split /\s+/,$line,2 ;
 
-	my $step = ($k =~ s/^\$//) ? $k 
-                 : ($v =~ m!://!)  ? "distributions:".$k
-                 :                 $k ; # old style parameter
-	my $leaf = $self->node->grab(step => $step) ;
-	$leaf->store($v) ;
-	$leaf->annotation($note) ;
+        my $step = ($k =~ s/^\$//) ? $k
+            : ($v =~ m!://!)  ? "distributions:".$k
+            :                 $k ; # old style parameter
+        my $leaf = $self->node->grab(step => $step) ;
+        $leaf->store($v) ;
+        $leaf->annotation($note) ;
     }
 
     return 1;
@@ -74,12 +65,7 @@ sub write {
 
     $logger->info("writing config file $args{file}");
     my $node = $args{object} ;
-    my $ioh  = $args{io_handle} ;
-
-    $ioh->print("## This file was written by 'cme edit approx'\n");
-    $ioh->print("## You may modify the content of this file.\n\n");
-
-    $ioh->printf("# %s\n", $node->annotation) if $node->annotation;
+    my $res = $self->write_global_comment('#');
 
     # Using Config::Model::ObjTreeScanner would be overkill
     foreach my $elt ($node->get_element_name) {
@@ -90,8 +76,8 @@ sub write {
         my $v = $obj->fetch ;
 
         if (defined $v) {
-            $ioh->printf("# %s\n", $obj->annotation) if $obj->annotation;
-            $ioh->printf("\$%-10s %s\n\n",$elt,$v) ;
+            $res .= sprintf("# %s\n", $obj->annotation) if $obj->annotation;
+            $res .= sprintf("\$%-10s %s\n\n",$elt,$v) ;
         }
     }
 
@@ -100,18 +86,31 @@ sub write {
         my $d = $node->grab("distributions:$dname") ;
 
         my $note = $d->annotation;
-        $ioh->print("# $note\n") if $note;
-	$ioh->printf("%-10s %s\n",$dname,$d->fetch) ;
+        $res .= "# $note\n" if $note;
+        $res .= sprintf("%-10s %s\n",$dname,$d->fetch) ;
     }
-    return 1;
 
+    $args{file_path}->spew_utf8($res);
+    return 1;
 }
 
 1;
 
+# ABSTRACT: Read and write Approx configuration file
+
+__END__
+
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
-Config::Model::Backend::Approx - Approx configuration file editor
+Config::Model::Backend::Approx - Read and write Approx configuration file
+
+=head1 VERSION
+
+version 1.011
 
 =head1 SYNOPSIS
 
@@ -121,15 +120,14 @@ Config::Model::Backend::Approx - Approx configuration file editor
 
 This module provides a backend to read and write configuration files for Approx.
 
-
 =head1 Methods
 
-=head2 read (object => approx_root, io_handle => ...)
+=head2 read
 
 Read F<approx.conf> and load the data in the C<approx_root>
 configuration tree.
 
-=head2 write (object => approx_root, io_handle => ...)
+=head2 write
 
 Write data from the C<approx_root> configuration tree into
 F<approx.conf>.
@@ -137,3 +135,17 @@ F<approx.conf>.
 =head1 SEE ALSO
 
 L<cme>, L<Config::Model::Backend::Any>,
+
+=head1 AUTHOR
+
+Dominique Dumont
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is Copyright (c) 2015-2018 by Dominique Dumont.
+
+This is free software, licensed under:
+
+  The GNU Lesser General Public License, Version 2.1, February 1999
+
+=cut
